#include <glib-object.h>
#include <string.h>
#include <stdio.h>

#include "widget.h"
#include "group.h"
#include "xml.h"
#include "page.h"
#include "cb.h"
#include "prop-util.h"

enum {
	GROUP_PROP_EMPTY,
	GROUP_PROP_INFO
};

enum {
	UPDATE_PROP,
	LAST_SIGNAL
};

G_DEFINE_TYPE(WPSGroup, wps_group, G_TYPE_OBJECT);

static guint group_signals[LAST_SIGNAL] = { 0 };

void wps_group_add_widget(WPSGroup *group, WPSWidget *widget)
{	
	
	if (GTK_IS_WIDGET(widget->object))
		widget->goo_item = goo_canvas_widget_new((GooCanvasItem *)group->goo_item,
							(GtkWidget*)widget->object,
							widget->widget_info->xalign,
							widget->widget_info->yalign,
							-1,
							-1,
							NULL);
	
	else if (GOO_IS_CANVAS_ITEM(widget->object))
		widget->goo_item = (GooCanvasItem *)widget->object;
		
	goo_canvas_item_add_child(group->goo_item, (GooCanvasItem *)widget->goo_item, -1);
	
}

static GObject *wps_group_constructor(GType type,
					guint n_construct_properties,
					GObjectConstructParam *construct_properties)
{
	GObject *obj;
	WPSGroup *group;
	GParameter *gparam;
	GObjectClass *object_class;
	GList *last_signal;
	GList *last_link_attr_info;
	LinkAttributeInfo *link_attr_info;
	
	obj = G_OBJECT_CLASS(wps_group_parent_class)->constructor(type, n_construct_properties, construct_properties);
	group = WPS_GROUP(obj);
		
	object_class = g_type_class_ref(wps_get_type_from_name("GooCanvasGroup"));
	gparam = set_gparam(object_class,
			group->group_info->prop_info_array,
			group->group_info->prop_len);
	
	group->goo_item = g_object_newv(wps_get_type_from_name("GooCanvasGroup"),
					group->group_info->prop_len,
					gparam);
					
	/* signals */	
	for (last_signal = g_list_last(group->group_info->signal_info); last_signal != NULL; last_signal = g_list_previous(last_signal)) {
		SignalInfo *signal = last_signal->data;
				
		add_widget_to_hash(signal->handler->str, (WPSWidget *)group);
	}	
	/* link attributes */
	
	if (group->group_info->link_attributes_info != NULL) {
		for (last_link_attr_info = g_list_last(group->group_info->link_attributes_info); last_link_attr_info != NULL; last_link_attr_info = g_list_previous(last_link_attr_info)) {
			LinkAttribute *link_attr;
			GValue *value;
			Tag *tag;
			
			link_attr_info = last_link_attr_info->data;
			link_attr = g_new0(LinkAttribute, 1);
			link_attr->name = g_strdup(link_attr_info->name->str);
			link_attr->tag = g_strdup(link_attr_info->tag->str);
			
			if (!g_strcmp0(link_attr_info->direction->str, "DIR_IN"))
				link_attr->direction = DIR_IN;
			else if (!g_strcmp0(link_attr_info->direction->str, "DIR_OUT"))
				link_attr->direction = DIR_OUT;
			else 
				g_warning("Unknown direction type\n");
			
			value = g_new0(GValue, 1);
			wps_widget_create_gvalue(link_attr_info, value);
			link_attr->value = value;
						
			tag_table_add_empty_key("test");
			tag = tag_table_find_tag_by_name(link_attr->tag);
			
			if (tag != NULL)
				link_attr->connected_tag = tag;
				
			group->link_attributes = g_list_append(group->link_attributes, link_attr);
		}
	}
					
	return obj;
}

static void wps_group_init(WPSGroup *group)
{
	group->name = NULL;
	group->group_info = NULL;
}


static void wps_group_get_property(	GObject *object,
					guint property_id,
					GValue *value,
					GParamSpec *pspec)
{
	WPSGroup *group = (WPSGroup *)object;
	
	switch (property_id) {
		case GROUP_PROP_INFO:
			g_value_set_pointer(value, group->group_info);
			break;
		default:
			break;
	}
}

static void wps_group_set_property(	GObject *object,
					guint property_id,
					const GValue *value,
					GParamSpec *pspec) 
{
	WPSGroup *group = (WPSGroup *)object;
	
	switch (property_id) {
		case GROUP_PROP_INFO:
			group->group_info = g_value_get_pointer(value);
			break;		
		default:
			break;
	}
}

static void wps_group_class_init(WPSGroupClass *klass)
{
	GObjectClass *object_class;
	
	object_class = G_OBJECT_CLASS(klass);	
	
	object_class->constructor = wps_group_constructor;
	object_class->get_property = wps_group_get_property;
	object_class->set_property = wps_group_set_property;
	
	klass->add_widget = wps_group_add_widget;
	klass->update_prop = NULL;	
	
	g_object_class_install_property(object_class,
					GROUP_PROP_INFO,
					g_param_spec_pointer("group_info", 
					"GroupInfo",
					"Group Info Structute",
					G_PARAM_READWRITE |
					G_PARAM_CONSTRUCT));
					
	group_signals[UPDATE_PROP] = g_signal_new ("update_prop",
						G_OBJECT_CLASS_TYPE (object_class),
						G_SIGNAL_RUN_FIRST,
						G_STRUCT_OFFSET (WPSGroupClass, update_prop),
						NULL, NULL,
						g_cclosure_marshal_VOID__VOID,
						G_TYPE_NONE, 0, NULL);
	
}

GList *wps_group_create_from_info(WPSPage *page, GList *group_info_list)
{
	GList *last_group = NULL;
	GList *group_list = NULL;
	GroupWidgetInfo *group_info;
	WPSGroup *group;
	
	for (last_group = g_list_last(group_info_list); last_group != NULL; last_group = g_list_previous(last_group)) {
		group_info = last_group->data;
		
		g_warning("%s\n", group_info->name->str);
		group = g_object_new(WPS_TYPE_GROUP, "group_info", group_info, NULL);
		group->widget_list = wps_widget_create_from_info((GObject *)group, group_info->widget_info);
		
		WPS_PAGE_GET_CLASS(page)->add_widget(page, (GObject *)group);
		group_list = g_list_append(group_list, group);
	}
	
	return group_list;
}

